#######################
# Built-in namespaces #
#######################

declare namespace xml      = "http://www.w3.org/XML/1998/namespace"
declare namespace xmlns    = "http://www.w3.org/2000/xmlns/"
declare namespace xsd      = "http://www.w3.org/2001/XMLSchema-datatypes"
declare namespace xs       = "http://www.w3.org/2001/XMLSchema"
declare namespace xsi      = "http://www.w3.org/2001/XMLSchema-instance"
declare namespace xf       = "http://www.w3.org/2002/08/xquery-functions"
declare namespace op       = "http://www.w3.org/2002/08/xquery-operators"
declare namespace xq       = "http://www.w3.org/2002/08/xquery"
declare namespace glx      = "http://db.bell-labs.com/galax"


#############################
# XML Schema wildcard types #
#############################

define type xs:AnyNumeric      { xsd:decimal | xsd:float | xsd:double | xsd:integer | xsd:int } 
 
define type xs:AnyScalar       { xsd:string |
			 	 xsd:boolean |
			 	 xsd:duration |
			 	 xsd:dateTime |
			 	 xsd:time |
			 	 xsd:date |
			 	 xsd:gYearMonth |
			 	 xsd:gYear |
			 	 xsd:gMonthDay |
			 	 xsd:gDay |
			 	 xsd:gMonth |
			 	 xsd:hexBinary |
			 	 xsd:base64Binary |
			 	 xsd:anyURI |
			 	 xsd:QName |
			 	 xsd:NOTATION |
			 	 type xs:AnyNumeric |
	                      	 xsd:UnknownSimpleType }

define type xs:AnySimpleType   { type xs:AnyScalar* }
define type xs:AnyAttribute    { attribute * { type xs:AnySimpleType } }
define type xs:AnyElement      { element * { type xs:AnyContentModel } }
define type xs:AnyContentModel { (type xs:AnyAttribute*, (type xs:AnySimpleType | (type xs:AnyElement | text)*)) }
define type xs:AnyDocument     { document { type xs:AnyNode* } }


#########################
# XQuery Wildcard types #
#########################

define type xs:AnyNode        { type xs:AnyDocument | type xs:AnyElement | type xs:AnyAttribute | text }
define type xs:AnyItem        { type xs:AnyNode | type xs:AnyScalar }
define type xs:AnyType        { type xs:AnyItem* }


#################
# F&O functions #
#################

#
# F&O Section 2. Accessors
#

define function signature  xf:node-kind(node)         returns (xsd:string)
define function signature  xf:node-name(node)         returns (xsd:QName?)
define function signature  xf:string(item?)           returns (xsd:string)
define function signature  xf:data(node)              returns (atomic*)
define function signature  xf:base-uri(node)          returns (xsd:anyURI?)


#
# F&O Section 5. Functions and Operators on Numbers
#

define function signature  op:numeric-add(numeric, numeric)      returns (numeric)
define function signature  op:numeric-subtract(numeric, numeric) returns (numeric)
define function signature  op:numeric-multiply(numeric, numeric) returns (numeric)
define function signature  op:numeric-divide(numeric, numeric)   returns (numeric)
define function signature  op:numeric-mod(numeric, numeric)      returns (numeric)

define function signature  op:numeric-unary-plus(numeric)        returns (numeric)
define function signature  op:numeric-unary-minus(numeric)       returns (numeric)

# Note: arithmetic functions are monomorphic in Galax for now...
# xsd:int
define function signature  op:int-add(xsd:int, xsd:int)      	 returns (xsd:int)
define function signature  op:int-subtract(xsd:int, xsd:int) 	 returns (xsd:int)
define function signature  op:int-multiply(xsd:int, xsd:int) 	 returns (xsd:int)
define function signature  op:int-divide(xsd:int, xsd:int)   	 returns (xsd:int)
define function signature  op:int-mod(xsd:int, xsd:int)      	 returns (xsd:int)

define function signature  op:int-unary-plus(xsd:int)    	 returns (xsd:int)
define function signature  op:int-unary-minus(xsd:int)   	 returns (xsd:int)

# xsd:integer
define function signature  op:integer-add(xsd:integer, xsd:integer)      returns (xsd:integer)
define function signature  op:integer-subtract(xsd:integer, xsd:integer) returns (xsd:integer)
define function signature  op:integer-multiply(xsd:integer, xsd:integer) returns (xsd:integer)
define function signature  op:integer-divide(xsd:integer, xsd:integer)   returns (xsd:integer)
define function signature  op:integer-mod(xsd:integer, xsd:integer)      returns (xsd:integer)

define function signature  op:integer-unary-plus(xsd:integer)    	 returns (xsd:integer)
define function signature  op:integer-unary-minus(xsd:integer)   	 returns (xsd:integer)

# xsd:decimal
define function signature  op:decimal-add(xsd:decimal, xsd:decimal)      returns (xsd:decimal)
define function signature  op:decimal-subtract(xsd:decimal, xsd:decimal) returns (xsd:decimal)
define function signature  op:decimal-multiply(xsd:decimal, xsd:decimal) returns (xsd:decimal)
define function signature  op:decimal-divide(xsd:decimal, xsd:decimal)   returns (xsd:decimal)
define function signature  op:decimal-mod(xsd:decimal, xsd:decimal)      returns (xsd:decimal)

define function signature  op:decimal-unary-plus(xsd:decimal)         returns (xsd:decimal)
define function signature  op:decimal-unary-minus(xsd:decimal)        returns (xsd:decimal)

# xsd:float
define function signature  op:float-add(xsd:float, xsd:float)         returns (xsd:float)
define function signature  op:float-subtract(xsd:float, xsd:float)    returns (xsd:float)
define function signature  op:float-multiply(xsd:float, xsd:float)    returns (xsd:float)
define function signature  op:float-divide(xsd:float, xsd:float)      returns (xsd:float)
define function signature  op:float-mod(xsd:float, xsd:float)         returns (xsd:float)

define function signature  op:float-unary-plus(xsd:float)    	      returns (xsd:float)
define function signature  op:float-unary-minus(xsd:float)   	      returns (xsd:float)

# xsd:double
define function signature  op:double-add(xsd:double, xsd:double)      returns (xsd:double)
define function signature  op:double-subtract(xsd:double, xsd:double) returns (xsd:double)
define function signature  op:double-multiply(xsd:double, xsd:double) returns (xsd:double)
define function signature  op:double-divide(xsd:double, xsd:double)   returns (xsd:double)
define function signature  op:double-mod(xsd:double, xsd:double)      returns (xsd:double)

define function signature  op:double-unary-plus(xsd:double)    	      returns (xsd:double)
define function signature  op:double-unary-minus(xsd:double)   	      returns (xsd:double)
       
define function signature  xf:floor(xsd:double)		       	      returns (xsd:int)
define function signature  xf:ceiling(xsd:double)              	      returns (xsd:int)
define function signature  xf:round(xsd:double)		       	      returns (xsd:int)


#
# F&O Section 6. Functions on Strings
#

# Note: This part of the F&O document has not been revised, and is
# probably sloppy

define function signature  xf:concat(xsd:string, xsd:string)                      returns (xsd:string)
define function signature  xf:concats(xsd:string*)                                returns (xsd:string)
define function signature  xf:starts-with(xsd:string, xsd:string)                 returns (xsd:boolean)
define function signature  xf:ends-with(xsd:string, xsd:string)                   returns (xsd:boolean)
define function signature  xf:contains(xsd:string, xsd:string)                    returns (xsd:boolean)
define function signature  xf:substring(xsd:string, xsd:int, xsd:int)             returns (xsd:string)
define function signature  xf:string-length(xsd:string)                           returns (xsd:int)
define function signature  xf:substring-before(xsd:string, xsd:string)            returns (xsd:string)
define function signature  xf:substring-after(xsd:string, xsd:string)             returns (xsd:string)
define function signature  xf:normalize-space(xsd:string)                         returns (xsd:string)
define function signature  xf:upper-case(xsd:string)                              returns (xsd:string)
define function signature  xf:lower-case(xsd:string)                              returns (xsd:string)
define function signature  xf:translate(xsd:string, xsd:string, xsd:string)       returns (xsd:string)
define function signature  xf:string-pad(xsd:string, xsd:int, xsd:string)         returns (xsd:string)


#
# F&O Section 7. Functions and Operators on Booleans
#

define function signature  xf:true()			      returns (xsd:boolean)
define function signature  xf:false()			      returns (xsd:boolean)

define function signature  op:or(xsd:boolean, xsd:boolean)    returns (xsd:boolean)
define function signature  op:and(xsd:boolean, xsd:boolean)   returns (xsd:boolean)
define function signature  xf:not(xsd:boolean)		      returns (xsd:boolean)


#
# F&O Section 8. Functions and Operators on Durations, Dates, and Times
#


#
# F&O Section 9. Functions on QNames
#

define function signature  xf:expanded-QName(xsd:string,xsd:string) returns (xsd:QName)
define function signature  xf:get-local-name-from-QName(xsd:QName)  returns (xsd:string?)
define function signature  xf:get-namespace-from-QName(xsd:QName?)  returns (xsd:anyURI?)


#
# F&O Section 10. Functions and Operators for anyURI
#

define function signature  xf:resolve-uri(xsd:anyURI, xsd:anyURI) returns (xsd:anyURI)


#
# F&O Section 11. Functions and Operators on base64Binary and hexBinary
#


#
# F&O Section 12. Functions and Operators on NOTATION
#


#
# F&O Section 13. Functions and Operators on Nodes
#

define function signature  xf:name(node?)                   returns (xsd:string)
define function signature  xf:local-name(node?)             returns (xsd:string)
define function signature  xf:namespace-uri(node?)          returns (xsd:string)
define function signature  op:node-equal(node, node)        returns (xsd:boolean)
define function signature  xf:deep-equal(node, node)        returns (xsd:boolean)
define function signature  op:node-before(node, node)       returns (xsd:boolean)
define function signature  op:node-after(node, node)        returns (xsd:boolean)
define function signature  xf:copy(node)                    returns (node)
define function signature  xf:root(node)                    returns (node)


#
# F&O Section 14. Functions and Operators on Sequences
#

define function signature  op:to(xsd:int, xsd:int)          returns (xsd:int+)

define function signature  xf:boolean(item*)                returns (xsd:boolean)
define function signature  xf:item-at(item*, xsd:int)       returns (item?)
define function signature  xf:index-of(node*, node)         returns (xsd:int)
define function signature  xf:empty(item*)            	    returns (xsd:boolean)
define function signature  xf:exists(item*)            	    returns (xsd:boolean)
define function signature  xf:distinct-nodes(node*)         returns (node*)
define function signature  xf:distinct-values(item*)  	    returns (item*)
define function signature  xf:insert(item*, xsd:int, item*) returns (item*)
define function signature  xf:remove(item*, xsd:int)               returns (item*)
define function signature  xf:subsequence(item*, xsd:int)          returns (item*)
define function signature  xf:subsequence(item*, xsd:int, xsd:int) returns (item*)

define function signature  xf:sequence-deep-equal(item*, item*) returns (xs:boolean)
define function signature  xf:sequence-node-equal(item*, item*) returns (xs:boolean)
define function signature  op:union(node*, node*)               returns (node*)
define function signature  op:intersect(node*, node*)           returns (node*)
define function signature  op:except(node*, node*)              returns (node*)

define function signature  xf:count(item*)                      returns (xsd:int)

#Note: aggregate functions are monomorphic in Galax for now...
# xsd:int
define function signature  xf:avg-int(xsd:int*)	              		    returns (xsd:int)
define function signature  xf:max-int(xsd:int*)	              		    returns (xsd:int?)
define function signature  xf:min-int(xsd:int*)	              		    returns (xsd:int?)
define function signature  xf:sum-int(xsd:int*)	              		    returns (xsd:int)
	      
# xsd:decimal
define function signature  xf:avg-decimal(xsd:decimal*)	              	    returns (xsd:decimal)
define function signature  xf:max-decimal(xsd:decimal*)	              	    returns (xsd:decimal?)
define function signature  xf:min-decimal(xsd:decimal*)	              	    returns (xsd:decimal?)
define function signature  xf:sum-decimal(xsd:decimal*)	              	    returns (xsd:decimal)
	      
# xsd:float
define function signature  xf:avg-float(xsd:float*)	      		    returns (xsd:float)
define function signature  xf:max-float(xsd:float*)	      		    returns (xsd:float?)
define function signature  xf:min-float(xsd:float*)	      		    returns (xsd:float?)
define function signature  xf:sum-float(xsd:float*)	      		    returns (xsd:float)

# xsd:double
define function signature  xf:avg-double(xsd:double*)	      		    returns (xsd:double)
define function signature  xf:max-double(xsd:double*)	      		    returns (xsd:double?)
define function signature  xf:min-double(xsd:double*)	      		    returns (xsd:double?)
define function signature  xf:sum-double(xsd:double*)	      		    returns (xsd:double)

# Note: input functions are not quite right yet... 
define function signature  xf:document(xsd:string)                          returns (document)


########################
# Comparison functions #
########################

# Note: Those functions are done in a polymorphic fashion in
# Galax. This differs from the F&O document where comparison functions
# are specialized by type.

# comparators

define function signature  op:equal(atomic, atomic)  returns (xsd:boolean)
define function signature  op:nequal(atomic, atomic) returns (xsd:boolean)
define function signature  op:ge(atomic, atomic)     returns (xsd:boolean)
define function signature  op:le(atomic, atomic)     returns (xsd:boolean)
define function signature  op:gt(atomic, atomic)     returns (xsd:boolean)
define function signature  op:lt(atomic, atomic)     returns (xsd:boolean)


###################
# Galax functions #
###################

# Note: Those functions are not part of the F&O spec. They are added
# in Galax for user's convenience.

# Sort by document order and duplicate removal

define function signature  glx:docorder(item*)                returns (item*)
define function signature  glx:distinct-docorder(item*)       returns (item*)

# "Type relief" operations				      
							      
define function signature  glx:unique(item*)                  returns (item)
define function signature  glx:optional(item*)                returns (item?)

# Union, intersection and exception on values

define function signature  glx:union-values(item*, item*)     returns (item*)
define function signature  glx:intersect-values(item*, item*) returns (item*)
define function signature  glx:except-values(item*, item*)    returns (item*)

# Other input functions

define function signature  glx:document-sax(xsd:string)                      returns (document)
define function signature  glx:document-validate(xsd:string, xsd:string)     returns (document)

